@@ -8,7 +8,7 @@ import (
 	"strings"
 	"sync"
 
-	vs "github.com/kubernetes-csi/external-snapshotter/v2/pkg/apis/volumesnapshot/v1beta1"
+	snapapi "github.com/kubernetes-csi/external-snapshotter/client/v4/apis/volumesnapshot/v1"
 	. "github.com/onsi/ginkgo" // nolint
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -28,7 +28,7 @@ var (
 	cephfsDeamonSetName   = "csi-cephfsplugin"
 	cephfsContainerName   = "csi-cephfsplugin"
 	cephfsDirPath         = "../deploy/cephfs/kubernetes/"
-	cephfsExamplePath     = "../examples/cephfs/"
+	cephfsExamplePath     = examplePath + "cephfs/"
 	subvolumegroup        = "e2e"
 	fileSystemName        = "myfs"
 )
@@ -55,33 +55,46 @@ func deployCephfsPlugin() {
 		e2elog.Failf("failed to delete nodeplugin rbac %s with error %v", cephfsDirPath+cephfsNodePluginRBAC, err)
 	}
 
-	createORDeleteCephfsResouces("create")
+	createORDeleteCephfsResources(kubectlCreate)
 }
 
 func deleteCephfsPlugin() {
-	createORDeleteCephfsResouces("delete")
+	createORDeleteCephfsResources(kubectlDelete)
 }
 
-func createORDeleteCephfsResouces(action string) {
+func createORDeleteCephfsResources(action kubectlAction) {
 	csiDriver, err := ioutil.ReadFile(cephfsDirPath + csiDriverObject)
 	if err != nil {
-		// createORDeleteRbdResouces is used for upgrade testing as csidriverObject is
+		// createORDeleteRbdResources is used for upgrade testing as csidriverObject is
 		// newly added, discarding file not found error.
 		if !os.IsNotExist(err) {
 			e2elog.Failf("failed to read content from %s with error %v", cephfsDirPath+csiDriverObject, err)
 		}
 	} else {
-		_, err = framework.RunKubectlInput(cephCSINamespace, string(csiDriver), action, "-f", "-")
+		err = retryKubectlInput(cephCSINamespace, action, string(csiDriver), deployTimeout)
 		if err != nil {
 			e2elog.Failf("failed to %s CSIDriver object with error %v", action, err)
 		}
 	}
+	cephConf, err := ioutil.ReadFile(examplePath + cephConfconfigMap)
+	if err != nil {
+		// createORDeleteCephfsResources is used for upgrade testing as cephConfConfigmap is
+		// newly added, discarding file not found error.
+		if !os.IsNotExist(err) {
+			e2elog.Failf("failed to read content from %s with error %v", examplePath+cephConfconfigMap, err)
+		}
+	} else {
+		err = retryKubectlInput(cephCSINamespace, action, string(cephConf), deployTimeout)
+		if err != nil {
+			e2elog.Failf("failed to %s ceph-conf configmap object with error %v", action, err)
+		}
+	}
 	data, err := replaceNamespaceInTemplate(cephfsDirPath + cephfsProvisioner)
 	if err != nil {
 		e2elog.Failf("failed to read content from %s with error %v", cephfsDirPath+cephfsProvisioner, err)
 	}
 	data = oneReplicaDeployYaml(data)
-	_, err = framework.RunKubectlInput(cephCSINamespace, data, action, ns, "-f", "-")
+	err = retryKubectlInput(cephCSINamespace, action, data, deployTimeout)
 	if err != nil {
 		e2elog.Failf("failed to %s CephFS provisioner with error %v", action, err)
 	}
@@ -90,7 +103,7 @@ func createORDeleteCephfsResouces(action string) {
 	if err != nil {
 		e2elog.Failf("failed to read content from %s with error %v", cephfsDirPath+cephfsProvisionerRBAC, err)
 	}
-	_, err = framework.RunKubectlInput(cephCSINamespace, data, action, ns, "-f", "-")
+	err = retryKubectlInput(cephCSINamespace, action, data, deployTimeout)
 	if err != nil {
 		e2elog.Failf("failed to %s CephFS provisioner rbac with error %v", action, err)
 	}
@@ -99,7 +112,7 @@ func createORDeleteCephfsResouces(action string) {
 	if err != nil {
 		e2elog.Failf("failed to read content from %s with error %v", cephfsDirPath+cephfsProvisionerPSP, err)
 	}
-	_, err = framework.RunKubectlInput(cephCSINamespace, data, action, ns, "-f", "-")
+	err = retryKubectlInput(cephCSINamespace, action, data, deployTimeout)
 	if err != nil {
 		e2elog.Failf("failed to %s CephFS provisioner psp with error %v", action, err)
 	}
@@ -108,7 +121,7 @@ func createORDeleteCephfsResouces(action string) {
 	if err != nil {
 		e2elog.Failf("failed to read content from %s with error %v", cephfsDirPath+cephfsNodePlugin, err)
 	}
-	_, err = framework.RunKubectlInput(cephCSINamespace, data, action, ns, "-f", "-")
+	err = retryKubectlInput(cephCSINamespace, action, data, deployTimeout)
 	if err != nil {
 		e2elog.Failf("failed to %s CephFS nodeplugin with error %v", action, err)
 	}
@@ -117,7 +130,7 @@ func createORDeleteCephfsResouces(action string) {
 	if err != nil {
 		e2elog.Failf("failed to read content from %s with error %v", cephfsDirPath+cephfsNodePluginRBAC, err)
 	}
-	_, err = framework.RunKubectlInput(cephCSINamespace, data, action, ns, "-f", "-")
+	err = retryKubectlInput(cephCSINamespace, action, data, deployTimeout)
 	if err != nil {
 		e2elog.Failf("failed to %s CephFS nodeplugin rbac with error %v", action, err)
 	}
@@ -126,7 +139,7 @@ func createORDeleteCephfsResouces(action string) {
 	if err != nil {
 		e2elog.Failf("failed to read content from %s with error %v", cephfsDirPath+cephfsNodePluginPSP, err)
 	}
-	_, err = framework.RunKubectlInput(cephCSINamespace, data, action, ns, "-f", "-")
+	err = retryKubectlInput(cephCSINamespace, action, data, deployTimeout)
 	if err != nil {
 		e2elog.Failf("failed to %s CephFS nodeplugin psp with error %v", action, err)
 	}
@@ -160,8 +173,12 @@ func validateSubvolumePath(f *framework.Framework, pvcName, pvcNamespace, fileSy
 		return err
 	}
 	if subVolumePath != subVolumePathInPV {
-		return fmt.Errorf("subvolumePath %s is not matching the subvolumePath %s in PV", subVolumePath, subVolumePathInPV)
+		return fmt.Errorf(
+			"subvolumePath %s is not matching the subvolumePath %s in PV",
+			subVolumePath,
+			subVolumePathInPV)
 	}
+
 	return nil
 }
 
@@ -214,7 +231,7 @@ var _ = Describe("cephfs", func() {
 		if CurrentGinkgoTestDescription().Failed {
 			// log pods created by helm chart
 			logsCSIPods("app=ceph-csi-cephfs", c)
-			// log provisoner
+			// log provisioner
 			logsCSIPods("app=csi-cephfsplugin-provisioner", c)
 			// log node plugin
 			logsCSIPods("app=csi-cephfsplugin", c)
@@ -226,11 +243,15 @@ var _ = Describe("cephfs", func() {
 		if err != nil {
 			e2elog.Failf("failed to delete configmap with error %v", err)
 		}
-		err = c.CoreV1().Secrets(cephCSINamespace).Delete(context.TODO(), cephFSProvisionerSecretName, metav1.DeleteOptions{})
+		err = c.CoreV1().
+			Secrets(cephCSINamespace).
+			Delete(context.TODO(), cephFSProvisionerSecretName, metav1.DeleteOptions{})
 		if err != nil {
 			e2elog.Failf("failed to delete provisioner secret with error %v", err)
 		}
-		err = c.CoreV1().Secrets(cephCSINamespace).Delete(context.TODO(), cephFSNodePluginSecretName, metav1.DeleteOptions{})
+		err = c.CoreV1().
+			Secrets(cephCSINamespace).
+			Delete(context.TODO(), cephFSNodePluginSecretName, metav1.DeleteOptions{})
 		if err != nil {
 			e2elog.Failf("failed to delete node secret with error %v", err)
 		}
@@ -273,6 +294,25 @@ var _ = Describe("cephfs", func() {
 				}
 			})
 
+			// test only if ceph-csi is deployed via helm
+			if helmTest {
+				By("verify PVC and app binding on helm installation", func() {
+					err := validatePVCAndAppBinding(pvcPath, appPath, f)
+					if err != nil {
+						e2elog.Failf("failed to validate CephFS pvc and application binding with error %v", err)
+					}
+					//  Deleting the storageclass and secret created by helm
+					err = deleteResource(cephfsExamplePath + "storageclass.yaml")
+					if err != nil {
+						e2elog.Failf("failed to delete CephFS storageclass with error %v", err)
+					}
+					err = deleteResource(cephfsExamplePath + "secret.yaml")
+					if err != nil {
+						e2elog.Failf("failed to delete CephFS storageclass with error %v", err)
+					}
+				})
+			}
+
 			By("check static PVC", func() {
 				scPath := cephfsExamplePath + "secret.yaml"
 				err := validateCephFsStaticPV(f, appPath, scPath)
@@ -298,7 +338,11 @@ var _ = Describe("cephfs", func() {
 
 			By("create PVC in storageClass with volumeNamePrefix", func() {
 				volumeNamePrefix := "foo-bar-"
-				err := createCephfsStorageClass(f.ClientSet, f, false, map[string]string{"volumeNamePrefix": volumeNamePrefix})
+				err := createCephfsStorageClass(
+					f.ClientSet,
+					f,
+					false,
+					map[string]string{"volumeNamePrefix": volumeNamePrefix})
 				if err != nil {
 					e2elog.Failf("failed to create storageclass with error %v", err)
 				}
@@ -324,6 +368,7 @@ var _ = Describe("cephfs", func() {
 					fmt.Printf("Checking prefix on %s\n", subVol)
 					if strings.HasPrefix(subVol.Name, volumeNamePrefix) {
 						foundIt = true
+
 						break
 					}
 				}
@@ -512,7 +557,7 @@ var _ = Describe("cephfs", func() {
 				if err != nil {
 					e2elog.Failf("failed to delete storageclass with error %v", err)
 				}
-				// verify subvolumegroup creation.
+				// verify subvolume group creation.
 				err = validateSubvolumegroup(f, "subvolgrp1")
 				if err != nil {
 					e2elog.Failf("failed to validate subvolume group with error %v", err)
@@ -591,7 +636,11 @@ var _ = Describe("cephfs", func() {
 				}
 
 				filePath := app.Spec.Containers[0].VolumeMounts[0].MountPath + "/test"
-				_, stdErr := execCommandInPodAndAllowFail(f, fmt.Sprintf("echo 'Hello World' > %s", filePath), app.Namespace, &opt)
+				_, stdErr := execCommandInPodAndAllowFail(
+					f,
+					fmt.Sprintf("echo 'Hello World' > %s", filePath),
+					app.Namespace,
+					&opt)
 				readOnlyErr := fmt.Sprintf("cannot create %s: Read-only file system", filePath)
 				if !strings.Contains(stdErr, readOnlyErr) {
 					e2elog.Failf(stdErr)
@@ -788,11 +837,11 @@ var _ = Describe("cephfs", func() {
 					snap.Spec.Source.PersistentVolumeClaimName = &pvc.Name
 					// create snapshot
 					for i := 0; i < totalCount; i++ {
-						go func(w *sync.WaitGroup, n int, s vs.VolumeSnapshot) {
+						go func(n int, s snapapi.VolumeSnapshot) {
 							s.Name = fmt.Sprintf("%s%d", f.UniqueName, n)
 							wgErrs[n] = createSnapshot(&s, deployTimeout)
-							w.Done()
-						}(&wg, i, snap)
+							wg.Done()
+						}(i, snap)
 					}
 					wg.Wait()
 
@@ -823,8 +872,7 @@ var _ = Describe("cephfs", func() {
 					// create multiple PVC from same snapshot
 					wg.Add(totalCount)
 					for i := 0; i < totalCount; i++ {
-
-						go func(w *sync.WaitGroup, n int, p v1.PersistentVolumeClaim, a v1.Pod) {
+						go func(n int, p v1.PersistentVolumeClaim, a v1.Pod) {
 							name := fmt.Sprintf("%s%d", f.UniqueName, n)
 							wgErrs[n] = createPVCAndApp(name, f, &p, &a, deployTimeout)
 							if wgErrs[n] == nil {
@@ -833,8 +881,8 @@ var _ = Describe("cephfs", func() {
 									wgErrs[n] = err
 								}
 							}
-							w.Done()
-						}(&wg, i, *pvcClone, *appClone)
+							wg.Done()
+						}(i, *pvcClone, *appClone)
 					}
 					wg.Wait()
 
@@ -854,12 +902,12 @@ var _ = Describe("cephfs", func() {
 					wg.Add(totalCount)
 					// delete clone and app
 					for i := 0; i < totalCount; i++ {
-						go func(w *sync.WaitGroup, n int, p v1.PersistentVolumeClaim, a v1.Pod) {
+						go func(n int, p v1.PersistentVolumeClaim, a v1.Pod) {
 							name := fmt.Sprintf("%s%d", f.UniqueName, n)
 							p.Spec.DataSource.Name = name
 							wgErrs[n] = deletePVCAndApp(name, f, &p, &a)
-							w.Done()
-						}(&wg, i, *pvcClone, *appClone)
+							wg.Done()
+						}(i, *pvcClone, *appClone)
 					}
 					wg.Wait()
 
@@ -876,11 +924,11 @@ var _ = Describe("cephfs", func() {
 
 					parentPVCCount := totalSubvolumes - totalCount
 					validateSubvolumeCount(f, parentPVCCount, fileSystemName, subvolumegroup)
-					// create clones from different snapshosts and bind it to an
+					// create clones from different snapshots and bind it to an
 					// app
 					wg.Add(totalCount)
 					for i := 0; i < totalCount; i++ {
-						go func(w *sync.WaitGroup, n int, p v1.PersistentVolumeClaim, a v1.Pod) {
+						go func(n int, p v1.PersistentVolumeClaim, a v1.Pod) {
 							name := fmt.Sprintf("%s%d", f.UniqueName, n)
 							p.Spec.DataSource.Name = name
 							wgErrs[n] = createPVCAndApp(name, f, &p, &a, deployTimeout)
@@ -890,8 +938,8 @@ var _ = Describe("cephfs", func() {
 									wgErrs[n] = err
 								}
 							}
-							w.Done()
-						}(&wg, i, *pvcClone, *appClone)
+							wg.Done()
+						}(i, *pvcClone, *appClone)
 					}
 					wg.Wait()
 
@@ -911,11 +959,11 @@ var _ = Describe("cephfs", func() {
 					wg.Add(totalCount)
 					// delete snapshot
 					for i := 0; i < totalCount; i++ {
-						go func(w *sync.WaitGroup, n int, s vs.VolumeSnapshot) {
+						go func(n int, s snapapi.VolumeSnapshot) {
 							s.Name = fmt.Sprintf("%s%d", f.UniqueName, n)
 							wgErrs[n] = deleteSnapshot(&s, deployTimeout)
-							w.Done()
-						}(&wg, i, snap)
+							wg.Done()
+						}(i, snap)
 					}
 					wg.Wait()
 
@@ -933,12 +981,12 @@ var _ = Describe("cephfs", func() {
 					wg.Add(totalCount)
 					// delete clone and app
 					for i := 0; i < totalCount; i++ {
-						go func(w *sync.WaitGroup, n int, p v1.PersistentVolumeClaim, a v1.Pod) {
+						go func(n int, p v1.PersistentVolumeClaim, a v1.Pod) {
 							name := fmt.Sprintf("%s%d", f.UniqueName, n)
 							p.Spec.DataSource.Name = name
 							wgErrs[n] = deletePVCAndApp(name, f, &p, &a)
-							w.Done()
-						}(&wg, i, *pvcClone, *appClone)
+							wg.Done()
+						}(i, *pvcClone, *appClone)
 					}
 					wg.Wait()
 
@@ -1014,11 +1062,11 @@ var _ = Describe("cephfs", func() {
 					wg.Add(totalCount)
 					// create clone and bind it to an app
 					for i := 0; i < totalCount; i++ {
-						go func(w *sync.WaitGroup, n int, p v1.PersistentVolumeClaim, a v1.Pod) {
+						go func(n int, p v1.PersistentVolumeClaim, a v1.Pod) {
 							name := fmt.Sprintf("%s%d", f.UniqueName, n)
 							wgErrs[n] = createPVCAndApp(name, f, &p, &a, deployTimeout)
-							w.Done()
-						}(&wg, i, *pvcClone, *appClone)
+							wg.Done()
+						}(i, *pvcClone, *appClone)
 					}
 					wg.Wait()
 
@@ -1045,12 +1093,12 @@ var _ = Describe("cephfs", func() {
 					wg.Add(totalCount)
 					// delete clone and app
 					for i := 0; i < totalCount; i++ {
-						go func(w *sync.WaitGroup, n int, p v1.PersistentVolumeClaim, a v1.Pod) {
+						go func(n int, p v1.PersistentVolumeClaim, a v1.Pod) {
 							name := fmt.Sprintf("%s%d", f.UniqueName, n)
 							p.Spec.DataSource.Name = name
 							wgErrs[n] = deletePVCAndApp(name, f, &p, &a)
-							w.Done()
-						}(&wg, i, *pvcClone, *appClone)
+							wg.Done()
+						}(i, *pvcClone, *appClone)
 					}
 					wg.Wait()
 
@@ -1110,7 +1158,11 @@ var _ = Describe("cephfs", func() {
 				}
 
 				filePath := app.Spec.Containers[0].VolumeMounts[0].MountPath + "/test"
-				_, stdErr := execCommandInPodAndAllowFail(f, fmt.Sprintf("echo 'Hello World' > %s", filePath), app.Namespace, &opt)
+				_, stdErr := execCommandInPodAndAllowFail(
+					f,
+					fmt.Sprintf("echo 'Hello World' > %s", filePath),
+					app.Namespace,
+					&opt)
 				readOnlyErr := fmt.Sprintf("cannot create %s: Read-only file system", filePath)
 				if !strings.Contains(stdErr, readOnlyErr) {
 					e2elog.Failf(stdErr)
@@ -1146,8 +1198,6 @@ var _ = Describe("cephfs", func() {
 			if err != nil {
 				e2elog.Failf("failed to delete user %s with error %v", keyringCephFSNodePluginUsername, err)
 			}
-
 		})
 	})
-
 })
