@@ -23,6 +23,7 @@ import (
 
 	"github.com/ceph/ceph-csi/internal/journal"
 	"github.com/ceph/ceph-csi/internal/util"
+	"github.com/ceph/ceph-csi/internal/util/log"
 )
 
 const (
@@ -119,7 +120,11 @@ deletion are inverse of each other, and protected by the request name lock, and
 hence any stale omaps are leftovers from incomplete transactions and are hence
 safe to garbage collect.
 */
-func checkSnapCloneExists(ctx context.Context, parentVol *rbdVolume, rbdSnap *rbdSnapshot, cr *util.Credentials) (bool, error) {
+func checkSnapCloneExists(
+	ctx context.Context,
+	parentVol *rbdVolume,
+	rbdSnap *rbdSnapshot,
+	cr *util.Credentials) (bool, error) {
 	err := validateRbdSnap(rbdSnap)
 	if err != nil {
 		return false, err
@@ -163,12 +168,14 @@ func checkSnapCloneExists(ctx context.Context, parentVol *rbdVolume, rbdSnap *rb
 			err = parentVol.deleteSnapshot(ctx, rbdSnap)
 			if err != nil {
 				if !errors.Is(err, ErrSnapNotFound) {
-					util.ErrorLog(ctx, "failed to delete snapshot %s: %v", rbdSnap, err)
+					log.ErrorLog(ctx, "failed to delete snapshot %s: %v", rbdSnap, err)
+
 					return false, err
 				}
 			}
-			err = undoSnapshotCloning(ctx, vol, rbdSnap, vol, cr)
+			err = undoSnapshotCloning(ctx, parentVol, rbdSnap, vol, cr)
 		}
+
 		return false, err
 	}
 
@@ -192,8 +199,9 @@ func checkSnapCloneExists(ctx context.Context, parentVol *rbdVolume, rbdSnap *rb
 		// create snapshot
 		sErr := vol.createSnapshot(ctx, rbdSnap)
 		if sErr != nil {
-			util.ErrorLog(ctx, "failed to create snapshot %s: %v", rbdSnap, sErr)
-			err = undoSnapshotCloning(ctx, vol, rbdSnap, vol, cr)
+			log.ErrorLog(ctx, "failed to create snapshot %s: %v", rbdSnap, sErr)
+			err = undoSnapshotCloning(ctx, parentVol, rbdSnap, vol, cr)
+
 			return false, err
 		}
 	}
@@ -204,20 +212,23 @@ func checkSnapCloneExists(ctx context.Context, parentVol *rbdVolume, rbdSnap *rb
 	if vol.ImageID == "" {
 		sErr := vol.getImageID()
 		if sErr != nil {
-			util.ErrorLog(ctx, "failed to get image id %s: %v", vol, sErr)
-			err = undoSnapshotCloning(ctx, vol, rbdSnap, vol, cr)
+			log.ErrorLog(ctx, "failed to get image id %s: %v", vol, sErr)
+			err = undoSnapshotCloning(ctx, parentVol, rbdSnap, vol, cr)
+
 			return false, err
 		}
 		sErr = j.StoreImageID(ctx, vol.JournalPool, vol.ReservedID, vol.ImageID)
 		if sErr != nil {
-			util.ErrorLog(ctx, "failed to store volume id %s: %v", vol, sErr)
-			err = undoSnapshotCloning(ctx, vol, rbdSnap, vol, cr)
+			log.ErrorLog(ctx, "failed to store volume id %s: %v", vol, sErr)
+			err = undoSnapshotCloning(ctx, parentVol, rbdSnap, vol, cr)
+
 			return false, err
 		}
 	}
 
-	util.DebugLog(ctx, "found existing image (%s) with name (%s) for request (%s)",
+	log.DebugLog(ctx, "found existing image (%s) with name (%s) for request (%s)",
 		rbdSnap.VolID, rbdSnap.RbdSnapName, rbdSnap.RequestName)
+
 	return true, nil
 }
 
@@ -278,7 +289,7 @@ func (rv *rbdVolume) Exists(ctx context.Context, parentVol *rbdVolume) (bool, er
 
 	// NOTE: Return volsize should be on-disk volsize, not request vol size, so
 	// save it for size checks before fetching image data
-	requestSize := rv.VolSize
+	requestSize := rv.VolSize //nolint:ifshort // FIXME: rename and split function into helpers
 	// Fetch on-disk image attributes and compare against request
 	err = rv.getImageInfo()
 	if err != nil {
@@ -295,12 +306,15 @@ func (rv *rbdVolume) Exists(ctx context.Context, parentVol *rbdVolume) (bool, er
 			}
 			err = j.UndoReservation(ctx, rv.JournalPool, rv.Pool,
 				rv.RbdImageName, rv.RequestName)
+
 			return false, err
 		}
+
 		return false, err
 	}
+	// TODO: check image needs flattening and completed?
 
-	err = rv.repairImageID(ctx, j)
+	err = rv.repairImageID(ctx, j, false)
 	if err != nil {
 		return false, err
 	}
@@ -320,14 +334,15 @@ func (rv *rbdVolume) Exists(ctx context.Context, parentVol *rbdVolume) (bool, er
 	}
 
 	if parentVol != nil && parentVol.isEncrypted() {
-		err = parentVol.copyEncryptionConfig(&rv.rbdImage)
+		err = parentVol.copyEncryptionConfig(&rv.rbdImage, false)
 		if err != nil {
-			util.ErrorLog(ctx, err.Error())
+			log.ErrorLog(ctx, err.Error())
+
 			return false, err
 		}
 	}
 
-	util.DebugLog(ctx, "found existing volume (%s) with image name (%s) for request (%s)",
+	log.DebugLog(ctx, "found existing volume (%s) with image name (%s) for request (%s)",
 		rv.VolID, rv.RbdImageName, rv.RequestName)
 
 	return true, nil
@@ -336,19 +351,27 @@ func (rv *rbdVolume) Exists(ctx context.Context, parentVol *rbdVolume) (bool, er
 // repairImageID checks if rv.ImageID is already available (if so, it was
 // fetched from the journal), in case it is missing, the imageID is obtained
 // and stored in the journal.
-func (rv *rbdVolume) repairImageID(ctx context.Context, j *journal.Connection) error {
+// if the force is set to true, the latest imageID will get added/updated in OMAP.
+func (rv *rbdVolume) repairImageID(ctx context.Context, j *journal.Connection, force bool) error {
+	if force {
+		// reset the imageID so that we can fetch latest imageID from ceph cluster.
+		rv.ImageID = ""
+	}
+
 	if rv.ImageID != "" {
 		return nil
 	}
 
 	err := rv.getImageID()
 	if err != nil {
-		util.ErrorLog(ctx, "failed to get image id %s: %v", rv, err)
+		log.ErrorLog(ctx, "failed to get image id %s: %v", rv, err)
+
 		return err
 	}
 	err = j.StoreImageID(ctx, rv.JournalPool, rv.ReservedID, rv.ImageID)
 	if err != nil {
-		util.ErrorLog(ctx, "failed to store volume id %s: %v", rv, err)
+		log.ErrorLog(ctx, "failed to store volume id %s: %v", rv, err)
+
 		return err
 	}
 
@@ -358,9 +381,7 @@ func (rv *rbdVolume) repairImageID(ctx context.Context, j *journal.Connection) e
 // reserveSnap is a helper routine to request a rbdSnapshot name reservation and generate the
 // volume ID for the generated name.
 func reserveSnap(ctx context.Context, rbdSnap *rbdSnapshot, rbdVol *rbdVolume, cr *util.Credentials) error {
-	var (
-		err error
-	)
+	var err error
 
 	journalPoolID, imagePoolID, err := util.GetPoolIDs(ctx, rbdSnap.Monitors, rbdSnap.JournalPool, rbdSnap.Pool, cr)
 	if err != nil {
@@ -391,7 +412,7 @@ func reserveSnap(ctx context.Context, rbdSnap *rbdSnapshot, rbdVol *rbdVolume, c
 		return err
 	}
 
-	util.DebugLog(ctx, "generated Volume ID (%s) and image name (%s) for request name (%s)",
+	log.DebugLog(ctx, "generated Volume ID (%s) and image name (%s) for request name (%s)",
 		rbdSnap.VolID, rbdSnap.RbdSnapName, rbdSnap.RequestName)
 
 	return nil
@@ -431,9 +452,7 @@ func updateTopologyConstraints(rbdVol *rbdVolume, rbdSnap *rbdSnapshot) error {
 // reserveVol is a helper routine to request a rbdVolume name reservation and generate the
 // volume ID for the generated name.
 func reserveVol(ctx context.Context, rbdVol *rbdVolume, rbdSnap *rbdSnapshot, cr *util.Credentials) error {
-	var (
-		err error
-	)
+	var err error
 
 	err = updateTopologyConstraints(rbdVol, rbdSnap)
 	if err != nil {
@@ -469,7 +488,7 @@ func reserveVol(ctx context.Context, rbdVol *rbdVolume, rbdSnap *rbdSnapshot, cr
 		return err
 	}
 
-	util.DebugLog(ctx, "generated Volume ID (%s) and image name (%s) for request name (%s)",
+	log.DebugLog(ctx, "generated Volume ID (%s) and image name (%s) for request name (%s)",
 		rbdVol.VolID, rbdVol.RbdImageName, rbdVol.RequestName)
 
 	return nil
@@ -510,6 +529,9 @@ func undoVolReservation(ctx context.Context, rbdVol *rbdVolume, cr *util.Credent
 // complete omap mapping between imageName and volumeID.
 
 // RegenerateJournal performs below operations
+// Extract clusterID, Mons after checkig clusterID mapping
+// Extract parameters journalPool, pool from volumeAttributes
+// Extract optional parameters volumeNamePrefix, kmsID, owner from volumeAttributes
 // Extract information from volumeID
 // Get pool ID from pool name
 // Extract uuid from volumeID
@@ -517,44 +539,50 @@ func undoVolReservation(ctx context.Context, rbdVol *rbdVolume, cr *util.Credent
 // Generate new volume Handler
 // The volume handler won't remain same as its contains poolID,clusterID etc
 // which are not same across clusters.
-func RegenerateJournal(imageName, volumeID, pool, journalPool, requestName string, cr *util.Credentials) (string, error) {
+func RegenerateJournal(
+	volumeAttributes map[string]string,
+	volumeID, requestName string,
+	cr *util.Credentials) (string, error) {
 	ctx := context.Background()
 	var (
-		options map[string]string
-		vi      util.CSIIdentifier
-		rbdVol  *rbdVolume
+		vi     util.CSIIdentifier
+		rbdVol *rbdVolume
+		kmsID  string
+		err    error
+		ok     bool
 	)
 
-	options = make(map[string]string)
 	rbdVol = &rbdVolume{}
 	rbdVol.VolID = volumeID
 
-	err := vi.DecomposeCSIID(rbdVol.VolID)
+	err = vi.DecomposeCSIID(rbdVol.VolID)
 	if err != nil {
 		return "", fmt.Errorf("%w: error decoding volume ID (%s) (%s)",
 			ErrInvalidVolID, err, rbdVol.VolID)
 	}
 
-	// TODO check clusterID mapping exists
-	rbdVol.ClusterID = vi.ClusterID
-	options["clusterID"] = rbdVol.ClusterID
-
-	rbdVol.Monitors, _, err = util.GetMonsAndClusterID(options)
+	kmsID, err = rbdVol.ParseEncryptionOpts(ctx, volumeAttributes)
 	if err != nil {
-		util.ErrorLog(ctx, "failed getting mons (%s)", err)
 		return "", err
 	}
 
-	rbdVol.Pool = pool
+	rbdVol.Monitors, rbdVol.ClusterID, err = util.FetchMappedClusterIDAndMons(ctx, vi.ClusterID)
+	if err != nil {
+		return "", err
+	}
+
+	if rbdVol.Pool, ok = volumeAttributes["pool"]; !ok {
+		return "", errors.New("required 'pool' parameter missing in volume attributes")
+	}
 	err = rbdVol.Connect(cr)
 	if err != nil {
 		return "", err
 	}
-	rbdVol.JournalPool = journalPool
+	rbdVol.JournalPool = volumeAttributes["journalPool"]
 	if rbdVol.JournalPool == "" {
 		rbdVol.JournalPool = rbdVol.Pool
 	}
-	volJournal = journal.NewCSIVolumeJournal("default")
+	volJournal = journal.NewCSIVolumeJournal(CSIInstanceID)
 	j, err := volJournal.Connect(rbdVol.Monitors, rbdVol.RadosNamespace, cr)
 	if err != nil {
 		return "", err
@@ -567,9 +595,8 @@ func RegenerateJournal(imageName, volumeID, pool, journalPool, requestName strin
 	}
 
 	rbdVol.RequestName = requestName
-	// TODO add Nameprefix also
+	rbdVol.NamePrefix = volumeAttributes["volumeNamePrefix"]
 
-	kmsID := ""
 	imageData, err := j.CheckReservation(
 		ctx, rbdVol.JournalPool, rbdVol.RequestName, rbdVol.NamePrefix, "", kmsID)
 	if err != nil {
@@ -591,6 +618,7 @@ func RegenerateJournal(imageName, volumeID, pool, journalPool, requestName strin
 		if err != nil {
 			return "", err
 		}
+
 		return rbdVol.VolID, nil
 	}
 
@@ -606,7 +634,7 @@ func RegenerateJournal(imageName, volumeID, pool, journalPool, requestName strin
 			undoErr := j.UndoReservation(ctx, rbdVol.JournalPool, rbdVol.Pool,
 				rbdVol.RbdImageName, rbdVol.RequestName)
 			if undoErr != nil {
-				util.ErrorLog(ctx, "failed to undo reservation %s: %v", rbdVol, undoErr)
+				log.ErrorLog(ctx, "failed to undo reservation %s: %v", rbdVol, undoErr)
 			}
 		}
 	}()
@@ -616,7 +644,7 @@ func RegenerateJournal(imageName, volumeID, pool, journalPool, requestName strin
 		return "", err
 	}
 
-	util.DebugLog(ctx, "re-generated Volume ID (%s) and image name (%s) for request name (%s)",
+	log.DebugLog(ctx, "re-generated Volume ID (%s) and image name (%s) for request name (%s)",
 		rbdVol.VolID, rbdVol.RbdImageName, rbdVol.RequestName)
 	if rbdVol.ImageID == "" {
 		err = rbdVol.storeImageID(ctx, j)
@@ -632,13 +660,16 @@ func RegenerateJournal(imageName, volumeID, pool, journalPool, requestName strin
 func (rv *rbdVolume) storeImageID(ctx context.Context, j *journal.Connection) error {
 	err := rv.getImageID()
 	if err != nil {
-		util.ErrorLog(ctx, "failed to get image id %s: %v", rv, err)
+		log.ErrorLog(ctx, "failed to get image id %s: %v", rv, err)
+
 		return err
 	}
 	err = j.StoreImageID(ctx, rv.JournalPool, rv.ReservedID, rv.ImageID)
 	if err != nil {
-		util.ErrorLog(ctx, "failed to store volume id %s: %v", rv, err)
+		log.ErrorLog(ctx, "failed to store volume id %s: %v", rv, err)
+
 		return err
 	}
+
 	return nil
 }
